window.Backed = window.Backed || {};
window.Backed.PropertyStore = window.Backed.PropertyStore || new Map();
const render = window.Backed.Renderer;
var propertyMixin = (base => {
  return class PropertyMixin extends base {
    static get observedAttributes() {
      return Object.entries(this.properties).map(entry => {
        if (entry[1].reflect) {
          return entry[0];
        } else return null;
      });
    }
    get properties() {
      return customElements.get(this.localName).properties;
    }
    constructor() {
      super();
      if (this.properties) {
        for (const entry of Object.entries(this.properties)) {
          const { observer, reflect, renderer } = entry[1];
          if (observer || reflect || renderer) {
            if (renderer && !render) {
              console.warn('Renderer undefined');
            }
          }
          this.defineProperty(entry[0], entry[1]);
        }
      }
    }
    connectedCallback() {
      if (super.connectedCallback) super.connectedCallback();
      if (this.attributes) for (const attribute of this.attributes) {
        if (String(attribute.name).includes('on-')) {
          const fn = attribute.value;
          const name = attribute.name.replace('on-', '');
          this.addEventListener(String(name), event => {
            let target = event.path[0];
            while (!target.host) {
              target = target.parentNode;
            }
            if (target.host[fn]) {
              target.host[fn](event);
            }
          });
        }
      }
    }
    attributeChangedCallback(name, oldValue, newValue) {
      this[name] = newValue;
    }
    defineProperty(property = null, { strict = false, observer, reflect = false, renderer, value }) {
      Object.defineProperty(this, property, {
        set(value) {
          if (value === this[`___${property}`]) return;
          this[`___${property}`] = value;
          if (reflect) {
            if (value) this.setAttribute(property, String(value));else this.removeAttribute(property);
          }
          if (observer) {
            if (observer in this) this[observer]();else console.warn(`observer::${observer} undefined`);
          }
          if (renderer) {
            if (renderer in this) render(this[renderer](), this.shadowRoot);else console.warn(`renderer::${renderer} undefined`);
          }
        },
        get() {
          return this[`___${property}`];
        },
        configurable: strict ? false : true
      });
      const attr = this.getAttribute(property);
      this[property] = attr || this.hasAttribute(property) || value;
    }
  };
});

export default propertyMixin;
//# sourceMappingURL=property-mixin.js.map
